// IL_P2P_IA.ls
// VER:2.0.0 / 2002101901 / beta
// (c) Reiner Schug

@version 2.5
@script motion
@name Parent2Point_IA

@define ERR_CRITICAL "Critical error! No particles found. Please re-add all P2Pnt scripts in correct order."
@define ERR_BADCONFIG "Config Error. Please re-add this script."

thisObj;
frametime=1/(Scene().fps!=0 ? Scene().fps : 25);
part=nil;
partObj=nil;
partObjName="";
partNum=1;

partOK=false;
criticalError=false;
firstCall=true;
isLoaded=false;

bake=0;
afterBake=0;
bakeMod=1;
useRot=1;

chX,chY,chZ;
chH,chP,chB;

//offsetvec=(<0,0,0>);

create: obj 
{
	thisObj=Mesh(obj.name);

	ch=thisObj.firstChannel();
	while(ch)
	{
		if(ch.name=="Position.X")
			chX=ch;
		else if(ch.name=="Position.Y")
			chY=ch;
		else if(ch.name=="Position.Z")
			chZ=ch;
		else if(ch.name=="Rotation.H")
			chH=ch;
		else if(ch.name=="Rotation.P")
			chP=ch;
		else if(ch.name=="Rotation.B")
			chB=ch;
		ch=thisObj.nextChannel();
	}

	setdesc("Parent2Point_IA"); 
}


process: ma, frame, time
{
	if(firstCall)
	{
		// wait for object during sceneload
		if(!isLoaded) return;
		partOK=initParticles();
		firstCall=false;
		return;
	}

	if(partOK && !criticalError && !afterBake)
	{
		
		newpos=part[1].getParticle(partNum, PART_POSITION);
		newrot=partObj.getRotation(time);
		ma.set(POSITION, newpos);
		if(useRot)
			ma.set(ROTATION, newrot);
		/* 
		ppos = part[1].getParticle(partNum, PART_POSITION);
		ppos.x += offsetvec.x;
		ppos.y += offsetvec.y;
		ppos.z += offsetvec.z;
		ma.set(POSITION, ppos);
		*/
		if(bake && (time > 0.0))
		{
			ntime=time - frametime * bakeMod;
			chX.createKey(ntime, newpos.x);
			chY.createKey(ntime, newpos.y);
			chZ.createKey(ntime, newpos.z);
			if(useRot)
			{
				chH.createKey(ntime, newrot.x);
				chP.createKey(ntime, newrot.y);
				chB.createKey(ntime, newrot.z);
			}
						
			if(frame >= Scene().previewend)
			{
				bake=0;
				afterBake=1;
				//partOK=false;
				Pause();
				thisObj.select();
				PreviewFrameStep(1);
			}
		}
	}
}


options
{
	if(criticalError) error(ERR_CRITICAL);

	reqbegin("Parent2Point Motion");
	if(!partOK && !part)
		c1 = ctlmeshitems("Mesh Item", partObj);
	c2 = ctlinteger("Particle #", partNum);
    //c3 = ctlvector("Offset X,Y,Z ?", offsetvec);
	c98 = ctlcheckbox("Use Rotation Channels?", useRot);
	ctlsep();
	c99 = ctlcheckbox("Bake on OK",bake);
	ctlsep();
	ctlbutton("Clear Keys", 100, "bt_clearkeys");
    if (!reqpost()) return;

	partNum=getvalue(c2);
    if(!partOK)
	{
		partObj=getvalue(c1);
		if(!partObj) error("Invalid Mesh!");
		partObjName=partObj.name;
		partOK=initParticles();
		if(!partOK)
		{
			criticalError=true;
			error(ERR_CRITICAL);
		}
		isLoaded=true;
		firstCall=false;
	}
	else
	{
		if(partNum<1 || partNum>part[1].count)
		{
			partOK=false;
			criticalError=true;
			error(ERR_BADCONFIG);
		}
	}
	bake=getvalue(c99);
	useRot=getvalue(c98);
	//offsetvec=getvalue(c3);

    reqend();

	if(bake)
	{
		frametime=1/(Scene().fps!=0 ? Scene().fps : 25);
		Pause();
		info("Please disable/remove the script after baking.");
		partObj.select();
		PreviewFrameStep(bakeMod);
		GoToFrame(0);
		PlayForward();
		return;
	}

}


initParticles
{
	criticalError=true;
	if(!partObjName) return false; 

	partObj=Mesh(partObjName);
	if(!partObj) return false;

	part=Particle(partObj) || error(ERR_CRITICAL);
	if(!part) return false;

	if( part.size() != 1  ) return false;
	if(partNum<1 || partNum>part[1].count) return false;

	criticalError=false;
	afterBake=0;

	return true;
}


bt_clearkeys
{
	for(ci=chX.keyCount; ci>0; ci--)
		chX.deleteKey(chX.keys[ci]);

	for(ci=chY.keyCount; ci>0; ci--)
		chY.deleteKey(chY.keys[ci]);

	for(ci=chZ.keyCount; ci>0; ci--)
		chZ.deleteKey(chZ.keys[ci]);

	for(ci=chH.keyCount; ci>0; ci--)
		chH.deleteKey(chH.keys[ci]);

	for(ci=chP.keyCount; ci>0; ci--)
		chP.deleteKey(chP.keys[ci]);

	for(ci=chB.keyCount; ci>0; ci--)
		chB.deleteKey(chB.keys[ci]);

	afterBake=0;
}


load: what,io
{
    if(what == SCENEMODE)
    {
		line = io.read();
		if(line) 
		{
			items=parse("$$",line);
			partObjName=items[2];
		}
		line = io.read();
		if(line) 
		{
			items=parse("$$",line);
			partNum=integer(items[2]);
		}

		line = io.read();
		if(line) 
		{
			items=parse("$$",line);
			useRot=integer(items[2]);
		}
			/*
			line = io.read();
			items=parse("$$",line);
			offsetvec.x=number(items[2]);

			line = io.read();
			items=parse("$$",line);
			offsetvec.y=number(items[2]);

			line = io.read();
			items=parse("$$",line);
			offsetvec.z=number(items[2]);
			*/

		isLoaded=true;
    }
}

save: what,io
{
	if(what == SCENEMODE)
    {
			io.writeln("partObjName$$", partObjName);
			io.writeln("partNum$$", partNum);
			io.writeln("useRot$$", useRot);
			/*
			io.writeln("offsetX$$",offsetvec.x);
			io.writeln("offsetY$$",offsetvec.y);
			io.writeln("offsetZ$$",offsetvec.z);
			*/
    }
}

